@@ -761,10 +761,10 @@ gb_eval_unary (GBEvalContext *ec,
 }
 
 GBValue *
-gb_eval_stream (GBEvalContext *ec,
-		const GBValue *v) {
+gb_eval_frx (GBEvalContext *ec,
+	     const GBValue *v) {
 
-	return gb_value_new_stream (v->v.s->str);
+	return gb_value_new_frx (v->v.s->str);
 }
 
 static GBValue *
@@ -838,7 +838,7 @@ gb_eval_context_eval (GBEvalContext *ec,
 		case GB_VALUE_STRING:
 			if ((e->parm.value->v.s->str[0] == '"')     &&
 			     strstr(e->parm.value->v.s->str, "\":")    ) {
-				val = gb_eval_stream (ec ,e->parm.value);
+				val = gb_eval_frx (ec ,e->parm.value);
 			} else
 				val = gb_value_copy (ec, e->parm.value);
 			break;
