@@ -14,17 +14,14 @@
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
-#include <unistd.h>
-#include <sys/mman.h>
-#include <sys/stat.h>
 
 #include "gb.h"
+#include "gb-frx.h"
 #include "gb-value.h"
 #include "gb-expr.h"
 #include "gb-object.h"
 #include "gb-eval.h"
-#include "gb-mmap-lex.h"
-#include "gb-stream.h"
+#include "gb-frx-get.h"
 
 /* Useful for using the standard comparison function */
 
@@ -40,55 +37,6 @@ GHashTable *type_hash     = NULL;
 GHashTable *type_def_hash = NULL;
 
 
-GBLexerStream *
-stream_from_file (const char *filename)
-{
-	int		 fd;
-	int		 len;
-
-	guint8		*data;
-	guint8		*datacpy;
-
-	struct stat	 sbuf;
-
-	GBLexerStream	*ans;
-
-	g_return_val_if_fail (filename != NULL, NULL);
-
-	fd = open (filename, O_RDONLY);
-	if (fd < 0 || fstat (fd, &sbuf) < 0) {
-		fprintf (stderr, "gb: %s : %s\n", filename, strerror (errno));
-
-		if (fd >= 0)
-			close (fd);
-		return NULL;
-	}
-
-	if ((len = sbuf.st_size) <= 0) {
-		fprintf (stderr, "%s : empty file\n", filename);
-		close (fd);
-		return NULL;
-	}
-
-	data = mmap (0, len, PROT_READ, MAP_PRIVATE, fd, 0);
-	if (data == MAP_FAILED) {
-		fprintf (stderr, "%s : unable to mmap\n", filename);
-		close (fd);
-		return NULL;
-	}
-
-	datacpy = g_new (guint8, len);
-	memcpy (datacpy, data, len);
-
-	ans = gb_mmap_stream_new (datacpy, len);
-
-	munmap ((char *)data, len);
-	close (fd);
-
-	return ans;
-}
-
-
 GBValue *
 gb_value_new_empty (void)
 {
@@ -293,17 +241,23 @@ gb_value_new_date_time (const GDate *date, int hr, int min, int sec)
 }
 
 
+/**
+ * gb_value_new_frx
+ *  @fn_offset	A string containing a filename in quotes and an offset.
+ *
+ **/
 GBValue *
-gb_value_new_stream (const char *fn_offset)
+gb_value_new_frx (const char *fn_offset)
 {
-	GBLexerStream	*ls;
 	GBValue		*v;
 	
 	gchar		*name;
 	gchar		**tokens;
 
 	guint32		 offset;
+	glong		 tmp;
 
+	
 	if (fn_offset != NULL) {
 		tokens = g_strsplit (fn_offset, ":", -1);
 		g_return_val_if_fail (tokens != NULL, NULL);	
@@ -311,37 +265,35 @@ gb_value_new_stream (const char *fn_offset)
 		name   = g_strdup (tokens[0]);
 		name   = g_strdup (&name[1]);
 		name   = g_strndup (name, strlen(name)-1);
-		offset = strtol (tokens[1], NULL, 16);
+		tmp    = strtol (tokens[1], NULL, 16);
 		g_strfreev (tokens);
 
-		if (offset == LONG_MIN) 
+		if (tmp == LONG_MIN) 
 			g_error ("Underflow processing %s\nError:%s\n", fn_offset,
 			         strerror (errno));
 
-		if (offset == LONG_MAX)
+		if (tmp < 0) 
+			g_error ("Negative number read when non-negative number expected processing %s\n", fn_offset);
+
+
+		if (tmp == LONG_MAX)
 			g_error ("Overflow processing %s\nError:%s\n", fn_offset,
 			         strerror (errno));
 
-		v                   = g_new (GBValue, 1);
-		v->type             = GB_VALUE_STREAM;
-		v->v.stream	    = g_new (GBFormStream, 1);
-		v->v.stream->offset = offset;
-		v->v.stream->s      = NULL;
+		offset = tmp;
 
-		v->v.stream->filename = g_strdup (name);
-		ls = stream_from_file (name);
-		if (!ls)
-			return NULL;
-
-		v->v.stream->s = ls;
+		v                  = g_new (GBValue, 1);
+		v->type            = GB_VALUE_FRX;
+		v->v.frx	   = g_new (GBFrx, 1);
+		v->v.frx->offset   = offset;
+		v->v.frx->filename = g_strdup (name);
 
 	} else {
-		v                     = g_new (GBValue, 1);
-		v->type               = GB_VALUE_STREAM;
-		v->v.stream	      = g_new (GBFormStream, 1);
-		v->v.stream->offset   = offset;
-		v->v.stream->s        = NULL;
-		v->v.stream->filename = NULL;
+		v                  = g_new (GBValue, 1);
+		v->type            = GB_VALUE_FRX;
+		v->v.frx	   = g_new (GBFrx, 1);
+		v->v.frx->offset   = offset;
+		v->v.frx->filename = NULL;
 	
 	}
 
@@ -1068,11 +1020,12 @@ gb_value_promote (GBEvalContext *ec,
 			
 		}
 		
-		case GB_VALUE_STREAM:
+		case GB_VALUE_FRX:
 		{
 			GPtrArray	*textptr;
 
-			textptr = gb_get_stream_text (v);
+
+			textptr = gb_get_frx_text (v);
 
 			ret = gb_value_new_string_chars (g_ptr_array_index (textptr, 0));
 			
@@ -1116,9 +1069,9 @@ gb_value_promote (GBEvalContext *ec,
 
 	case GB_VALUE_LIST:
 		switch (from) {
-		case GB_VALUE_STREAM:
+		case GB_VALUE_FRX:
 		{
-			ret = gb_value_new_list (gb_get_stream_list (v));
+			ret = gb_value_new_list (gb_get_frx_list (v));
 		
 			break;
 		}
@@ -1279,9 +1232,9 @@ gb_value_init (void)
 	add_type_map ("Variant", gb_value_new_int     (0));
 	/* Data object */
 	/* Decimal */
-	add_type_map ("Byte",    gb_value_new_byte    (0));
-	add_type_map ("Stream",  gb_value_new_stream  (NULL));
-	add_type_map ("List",    gb_value_new_list    (NULL));
+	add_type_map ("Byte",    gb_value_new_byte (0));
+	add_type_map ("Frx",     gb_value_new_frx  (NULL));
+	add_type_map ("List",    gb_value_new_list (NULL));
 }
 
 static void
