@@ -61,7 +61,7 @@ gbrun_eval_context_new (const char        *module_name,
 	ec->random.reseed = FALSE;
 	ec->random.randseed = 0;
 
-	ec->open_files = NULL;
+	gbrun_files_init (ec);
 
 	return ec;
 }
