@@ -15,6 +15,7 @@
 #include <gb/gb-value.h>
 #include <gb/gb-expr.h>
 #include <gb/gb-eval.h>
+#include <gbrun/gbrun-file.h>
 
 #define GBRUN_TYPE_EVAL_CONTEXT            (gbrun_eval_context_get_type ())
 #define GBRUN_EVAL_CONTEXT(obj)            (GTK_CHECK_CAST ((obj), GBRUN_TYPE_EVAL_CONTEXT, GBRunEvalContext))
@@ -35,7 +36,7 @@ struct _GBRunEvalContext {
 	GBValue          *me;
 
 	/* Runtime execution state storage */
-	GSList *open_files;
+	GBRunFileHandle  *file_handles;
 
         struct {
 		gint     randseed;
