@@ -3,8 +3,9 @@
  * GNOME Basic File I/O statements handler
  *
  * Authors:
- *    Ravi Pratap   (ravi_pratap@email.com)
- *    Michael Meeks (michael@helixcode.com)
+ *    Ravi Pratap    (ravi_pratap@email.com)
+ *    Michael Meeks  (michael@helixcode.com)
+ *    William Miller (William234@aol.com)
  *
  * Copyright 2000, Helix Code, Inc.
  */
@@ -24,36 +25,47 @@
 
 #define FILE_STMT_SEC_CHECK(ec,txt) \
 	if (ec->flags & GBRUN_SEC_IO) { \
-		gbrun_exception_firev (ec, "Insufficient privilage to %s file", \
+		gbrun_exception_firev (ec, "Insufficient privilege to %s file", \
 				       txt); \
 		return FALSE; \
 	}
 
-typedef struct {
+struct _GBRunFileHandle {
         /* VB related */
+        int       used;
 	int       fileno;
 	OpenMode  mode;
 	guint     recordlen;
 
 	/* Internal */
 	FILE     *file;
-} GBRunFileHandle;
+};
 
 static GBRunFileHandle *
 internal_handle_from_gb_no (GBRunEvalContext *ec, int fileno)
 {
-	GSList *l;
+	GBRunFileHandle *h;
+	GBRunFileHandle *handles;
 
-	for (l = ec->open_files; l != NULL; l = l->next) {
-		GBRunFileHandle *h = l->data;
+	handles = ec->file_handles;
 
-		if (h->fileno == fileno) 
-			return h;
+	if (!(fileno >= 0 && fileno <= 512)) {
+		gbrun_exception_fire (ec, "File number out of range");
+		return NULL;
 	}
-		
-	gbrun_exception_firev (ec, "Invalid filenumber %d", fileno);
 
-	return NULL;
+	if (handles [fileno].used == FALSE) {
+		gbrun_exception_firev (ec, "Invalid filenumber %d", fileno);
+		return NULL;
+	} else {
+		h = g_new (GBRunFileHandle, 1);
+		h->fileno = handles [fileno].fileno;
+		h->mode = handles [fileno].mode;
+		h->recordlen = handles [fileno].recordlen;
+		h->file = handles [fileno].file;
+	}
+
+	return h;
 }
 
 /********** File I/O functions ************/
@@ -63,16 +75,19 @@ gbrun_func_eof (GBRunEvalContext *ec,
 		GBObject         *object,
 		GBValue         **args)
 {
-	GBRunFileHandle *h;	
+	GBRunFileHandle *h;
 
 	GB_IS_VALUE (ec, args [0], GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
-	
-	if (feof (h->file))
+
+	if (feof (h->file)) {
+		g_free (h);
 		return gb_value_new_boolean (GBTrue);
-	else
+	} else {
+		g_free (h);
 		return gb_value_new_boolean (GBFalse);
+	}
 }
 
 GBValue *
@@ -80,33 +95,50 @@ gbrun_func_freefile (GBRunEvalContext *ec,
 		     GBObject         *object,
 		     GBValue         **args)
 {
-	GSList *l;
-	int     max = -1;
+	int              max  = -1;
+	int              arg  = 0;
+	int              loop = 0;
+	GBRunFileHandle *handles = ec->file_handles;
 
-	/*
-	 * FIXME: obey the range argument.
-	 */
-	if (args [0])
-		g_warning ("FreeFile: Rangenumber unimplemented");
-
-	/*
-	 * FIXME: need to test whether back to back calls to freefile
-	 * return the same number, or to avoid races, reserve the ID.
-	 */
-	for (l = ec->open_files; l != NULL; l = l->next) {
-		GBRunFileHandle *h = l->data;
-
-		/*
-		 * FIXME: this will die after too few file opens, we
-		 * need to sort by fileno and find non-adjacent ids.
-		 */
-
-		if (h->fileno > max)
-			max = h->fileno;
+	if (args [0]) {
+		arg = args[0]->v.i;
+	} else {
+		arg = 0;
 	}
 
-	if (max < 0)
-		max = 1;
+	switch (arg) {
+
+	/*
+	 * 0 - 255
+	 * Locked for exclusive use by the application
+	 */
+
+	case 0:
+	        for (loop = 0; loop < 256; loop++) {
+			if (handles [loop].used == FALSE) {
+				return gb_value_new_int (loop);
+				break;
+			}
+	  	}
+		break;
+
+ 	/*
+	 * 256 - 511
+	 * Not locked for exclusive use.
+ 	 */
+	case 1:
+	        for (loop = 256; loop < 512; loop++) {
+			if (handles [loop].used == FALSE) {
+				return gb_value_new_int (loop);
+				break;
+			}
+	  	}
+		break;
+
+	default:
+		g_warning("You need to specify either a 0 or a 1");
+		break;
+	}
 
 	return gb_value_new_int (max);
 }
@@ -119,25 +151,47 @@ gbrun_func_loc (GBRunEvalContext *ec,
 	GBRunFileHandle *h;
 	GBValue         *res;
 	long             pos;
-	
+
 	GB_IS_VALUE (ec, args [0], GB_VALUE_INT);
-	
+
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
 
 	pos = ftell (h->file) + 1;
 
 	if (h->mode == GB_OPEN_RANDOM)
 		res = gb_value_new_long (pos / h->recordlen);
-	else if (h->mode == GB_OPEN_INPUT  || 
+	else if (h->mode == GB_OPEN_INPUT  ||
 		 h->mode == GB_OPEN_OUTPUT ||
 		 h->mode == GB_OPEN_APPEND)
 		res = gb_value_new_long (pos / 128);
 	else if (h->mode == GB_OPEN_BINARY)
 		res = gb_value_new_long (pos);
 
+	g_free (h);
+
 	return res;
 }
 
+long
+get_file_len (FILE *fin)
+{
+	long  len;
+	long  pos;
+
+	pos = ftell (fin);
+	if (fseek (fin, 0, SEEK_END)) {
+		return -1;
+	}
+
+	len = ftell (fin);
+	if (fseek (fin, pos, SEEK_SET)) {
+		return -1;
+	}
+
+	return len;
+}
+
+
 GBValue *
 gbrun_func_lof (GBRunEvalContext *ec,
 		GBObject         *object,
@@ -145,30 +199,21 @@ gbrun_func_lof (GBRunEvalContext *ec,
 {
 	GBRunFileHandle *h;
 	GBValue         *res;
-	long             pos;
-	
+	long             len;
+
 	GB_IS_VALUE (ec, args[0], GB_VALUE_INT);
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
-	
-        /* Save current position */
-	pos = ftell (h->file); 
-	
-	/*
-	 * This small roundabout way of getting the size of an
-	 * open file is necessary as we have ostensibly, no func
-	 * which does that for us on an open file using the FILE * !
-	 */
-	if (fseek (h->file, 0, SEEK_END)) {
-		gbrun_exception_fire (ec, "Lof can't seek to EOF!");
-		return NULL;
-	}
-	
-	res = gb_value_new_long (ftell (h->file));
-	if (fseek (h->file, pos, SEEK_SET)) {
-		gbrun_exception_fire (ec, "Lof can't seek to EOF!");
+
+	len = get_file_len (h->file);
+
+	if (len != -1) {
+		res = gb_value_new_long (len);
+	} else {
+		gbrun_exception_fire (ec, "Unable to get the LOF.");
 		return NULL;
 	}
 
+	g_free (h);
 	return res;
 }
 
@@ -180,49 +225,51 @@ gbrun_func_seek (GBRunEvalContext *ec,
 	GBRunFileHandle *h;
 	GBValue         *res;
 	long             pos;
-	
+
 	GB_IS_VALUE (ec, args [0], GB_VALUE_INT);
-	
+
 	h = internal_handle_from_gb_no (ec, args [0]->v.i);
 
 	pos = ftell (h->file) + 1;
 
 	if (h->mode == GB_OPEN_RANDOM)
 		res = gb_value_new_long (pos / h->recordlen);
-	else 
+	else
 		res = gb_value_new_long (pos);
-	
+
+	g_free (h);
 	return res;
 }
 
 gboolean
 gbrun_files_clean (GBRunEvalContext *ec)
 {
-	GSList *l;
+	int              loop;
+	GBRunFileHandle *handles = ec->file_handles;
 
-	for (l = ec->open_files; l; l = l->next) {
-		GBRunFileHandle *h = l->data;
+	for (loop = 0; loop < 512; loop++) {
+		if (handles [loop].used == TRUE) {
+			fclose (handles [loop].file);
+		}
+ 	}
 
-		if (h->file)
-			fclose (h->file);
-		h->file = NULL;
-		g_free (h);
-	}
-	g_slist_free (ec->open_files);
-	ec->open_files = NULL;
+	g_free (ec->file_handles);
 
 	return TRUE;
 }
 
-gboolean 
+gboolean
 gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 {
-	GBRunFileHandle  *h = g_new (GBRunFileHandle, 1);
+	GBRunFileHandle  *h;
+	GBRunFileHandle  *handles = ec->file_handles;
 
 	GBValue  *filename  = gb_eval_context_eval (GB_EVAL_CONTEXT (ec), stmt->parm.open.filename);
 	GBValue  *handle    = gb_eval_context_eval (GB_EVAL_CONTEXT (ec), stmt->parm.open.handle);
 	GBValue  *reclen;
 	FILE     *fd;
+	int		 filedes;
+	int		 lock = 0;
 
 	FILE_STMT_SEC_CHECK (ec, "open");
 
@@ -232,31 +279,61 @@ gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 		return FALSE;
 	}
 
+	if (handles [handle->v.i].used) {
+		gbrun_exception_fire (ec, "File number already in use !");
+		return FALSE;
+	}
+
+	/*
+	 * If the file is opened with a filenumber between
+	 * 256 and 512, open it locked.
+	 */
+	if (handle->v.i >= 256 && handle->v.i <= 512) {
+		filedes = open (filename->v.s->str, O_CREAT);
+		lock = 1;
+	}
+
 	switch (stmt->parm.open.mode) {
-		
+
 	case GB_OPEN_OUTPUT:
-		fd = fopen (filename->v.s->str, "w");
+		if (lock != 1) {
+			fd = fopen (filename->v.s->str, "w");
+		} else {
+			fd = fdopen (filedes, "w");
+		}
 		break;
 
 	case GB_OPEN_INPUT:
-		fd = fopen (filename->v.s->str, "r");
+		if (lock != 1) {
+			fd = fopen (filename->v.s->str, "r");
+		} else {
+			fd = fdopen (filedes, "r");
+		}
 		break;
 
 	case GB_OPEN_APPEND:
-		fd = fopen (filename->v.s->str, "a");
+		if (lock != 1) {
+			fd = fopen (filename->v.s->str, "a");
+		} else {
+			fd - fdopen (filedes, "a");
+		}
 		break;
-	       
+
 	case GB_OPEN_BINARY: case GB_OPEN_RANDOM:
 		/*
 		 * Small workaround to create file if it doesn't
 		 * exist. Apparently, directly using ab+ doesn't allow
-		 * repositioning of stream : behaves funnily 
+		 * repositioning of stream : behaves funnily
 		 */
-		fd = fopen (filename->v.s->str, "ab+");
-		fclose (fd); 
-		fd = fopen (filename->v.s->str, "rb+");
+		if (lock != 1 ) {
+			fd = fopen (filename->v.s->str, "ab+");
+			fclose (fd);
+			fd = fopen (filename->v.s->str, "rb+");
+		} else {
+			fd = fdopen (filedes, "rb+");
+		}
 		break;
-		
+
 	default:
 		g_warning ("Unhandled OPEN mode");
 		break;
@@ -266,15 +343,18 @@ gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 		gbrun_exception_fire (ec, "Open failed");
 		return FALSE;
 	}
-	
+
+	h = &handles [handle->v.i];
+
 	h->fileno = handle->v.i;
 	h->mode   = stmt->parm.open.mode;
-	h->file   = fd;
+    h->file   = fd;
+	h->used   = TRUE;
 
 	if (stmt->parm.open.recordlen) {
 		reclen = gb_eval_context_eval (GB_EVAL_CONTEXT (ec), stmt->parm.open.recordlen);
 		h->recordlen = reclen->v.i;
-	} else 
+	} else
 		h->recordlen = 1;   /* Nothing specified for record length */
 
 	if (h->mode == GB_OPEN_BINARY)
@@ -283,16 +363,15 @@ gbrun_stmt_open (GBRunEvalContext *ec, const GBStatement *stmt)
 	/* Rewind file pointer to beginning always */
 	fseek (h->file, 0, SEEK_SET);
 
-	ec->open_files = g_slist_prepend (ec->open_files, (gpointer) h);
-	
 	return TRUE;
 }
 
-gboolean 
+gboolean
 gbrun_stmt_close (GBRunEvalContext *ec, const GBStatement *stmt)
 {
 	GSList *l;
-	
+	GBRunFileHandle *handles = ec->file_handles;
+
 	FILE_STMT_SEC_CHECK (ec, "close");
 
 	/* If no arguments are passed, close all open files */
@@ -308,18 +387,19 @@ gbrun_stmt_close (GBRunEvalContext *ec, const GBStatement *stmt)
 					GB_VALUE_INT);
 
 		h = internal_handle_from_gb_no (ec, fileno->v.i);
-		gb_value_destroy (fileno);
+
 
 		if (h) {
-			ec->open_files = g_slist_remove (ec->open_files, h);
-
 			fclose (h->file);
-			h->file = NULL;
 			g_free (h);
-		} else 
+			handles [fileno->v.i].used = FALSE;
+			handles [fileno->v.i].file = NULL;
+		} else
 			return FALSE;
+
+		gb_value_destroy (fileno);
 	}
-		
+
 	return TRUE;
 }
 
@@ -348,7 +428,7 @@ read_string (GBRunEvalContext *ec, GBRunFileHandle *h, gboolean allow_space)
 		chr = (char)c;
 		g_array_append_val (chars, chr);
 	}
-	
+
 	chr = '\0';
 	g_array_append_val (chars, chr);
 
@@ -359,14 +439,14 @@ read_string (GBRunEvalContext *ec, GBRunFileHandle *h, gboolean allow_space)
 	return val;
 }
 
-gboolean 
+gboolean
 gbrun_stmt_line_input (GBRunEvalContext *ec, const GBStatement *stmt)
 {
 	GBValue         *handle;
 	GBValue         *val;
 	const GBExpr    *objref = stmt->parm.line_input.objref;
 	GBRunFileHandle *h;
-	
+
 	FILE_STMT_SEC_CHECK (ec, "line input from");
 
 	handle = gbrun_eval_as (ec, stmt->parm.line_input.handle,
@@ -378,7 +458,7 @@ gbrun_stmt_line_input (GBRunEvalContext *ec, const GBStatement *stmt)
 	if (objref->type != GB_EXPR_OBJREF) {
 		gbrun_exception_fire (ec, "Need an object ref");
 		return FALSE;
-	} else if (h->mode != GB_OPEN_INPUT  && 
+	} else if (h->mode != GB_OPEN_INPUT  &&
 		   h->mode != GB_OPEN_RANDOM &&
 		   h->mode != GB_OPEN_BINARY) {
 		gbrun_exception_fire (ec, "Can't read from a file not opened for input/random/binary");
@@ -393,9 +473,10 @@ gbrun_stmt_line_input (GBRunEvalContext *ec, const GBStatement *stmt)
 		gbrun_exception_fire (ec, "Assignment of value to objectref failed");
 		return FALSE;
 	}
-	
+
 	g_free (val);
-		
+	g_free (h);
+
 	return TRUE;
 }
 
@@ -415,7 +496,7 @@ gbrun_stmt_input (GBRunEvalContext *ec, const GBStatement *stmt)
 	h = internal_handle_from_gb_no (ec, handle->v.i);
 	gb_value_destroy (handle);
 
-	if (h->mode != GB_OPEN_INPUT  && 
+	if (h->mode != GB_OPEN_INPUT  &&
 	    h->mode != GB_OPEN_RANDOM &&
 	    h->mode != GB_OPEN_BINARY) {
 		gbrun_exception_fire (ec, "Can't read from a file not opened for input/random/binary");
@@ -441,16 +522,19 @@ gbrun_stmt_input (GBRunEvalContext *ec, const GBStatement *stmt)
 		}
 	}
 
+	g_free (h);
 	return TRUE;
 }
 
-gboolean 
+gboolean
 gbrun_stmt_get (GBRunEvalContext *ec, const GBStatement *stmt)
 {
 	GBValue         *handle;
 	GBRunFileHandle *h;
 	GBValue         *recordnum = NULL;
-	
+	long  recl;
+	long  len;
+
 	FILE_STMT_SEC_CHECK (ec, "get from");
 
 	if (stmt->parm.get.recordnum)
@@ -474,9 +558,21 @@ gbrun_stmt_get (GBRunEvalContext *ec, const GBStatement *stmt)
 
 	/*
 	 * FIXME: need to check bounds on recordnum ideally.
+	 * The second IF should prevent trying to read past the
+	 * end of the file.
 	 */
-	if (h->recordlen && recordnum) 
-		fseek (h->file, (h->recordlen) * (recordnum->v.l - 1), SEEK_SET);
+
+	if (h->recordlen && recordnum) {
+		recl = h->recordlen * recordnum->v.l - 1;
+		len = get_file_len (h->file);
+
+		if (recl < len) {
+			fseek (h->file, recl, SEEK_SET);
+		} else {
+			gbrun_exception_fire (ec, "Attempted to read past end of file");
+			return FALSE;
+		}
+	}
 
 	if (!feof (h->file)) {
 		GBValue      *val;
@@ -484,25 +580,26 @@ gbrun_stmt_get (GBRunEvalContext *ec, const GBStatement *stmt)
 		char         *tmpstr = g_strdup ("");
 		const GBExpr *objref = stmt->parm.get.objref;
 		int          i;
-		
+
 		for (i = 0; i < h->recordlen; ++i) {
 			tmpstr[0] = (char) fgetc (h->file);
 			tmpstr[1] = '\0';
 			str = g_strconcat (str, tmpstr, NULL);
 		}
-		
+
 		val = gb_value_new_string_chars (str);
 
 		if (!gbrun_eval_assign (ec, objref->parm.objref, val)) {
 			gbrun_exception_fire (ec, "Assignment of value to objref failed");
 			return FALSE;
 		}
-		
+
 		gb_value_destroy (val);
 		g_free (str);
 		g_free (tmpstr);
-	} 
-	
+	}
+
+	g_free (h);
 	return TRUE;
 }
 
@@ -516,11 +613,11 @@ gbrun_stmt_put (GBRunEvalContext *ec, const GBStatement *stmt)
 
 	FILE_STMT_SEC_CHECK (ec, "put to");
 
-	if (stmt->parm.put.recordnum) 
+	if (stmt->parm.put.recordnum)
 		recordnum = gbrun_eval_as (ec, stmt->parm.put.recordnum,
 					   GB_VALUE_LONG);
-	
-	handle = gbrun_eval_as (ec, stmt->parm.put.handle,  
+
+	handle = gbrun_eval_as (ec, stmt->parm.put.handle,
 				GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, handle->v.i);
@@ -531,14 +628,14 @@ gbrun_stmt_put (GBRunEvalContext *ec, const GBStatement *stmt)
 		gbrun_exception_fire (ec, "Can't Put to a file not opened in random/binary");
 		return FALSE;
 	}
-	
+
 	if (h->mode == GB_OPEN_RANDOM)
 		g_warning ("Put: Can't handle Random mode correctly yet.");
 
 	/* FIXME: do we need to check ranges here ? */
 	if (h->recordlen && recordnum)
 		fseek (h->file, (h->recordlen) * (recordnum->v.l - 1), SEEK_SET);
-	
+
         tmp = gbrun_eval_objref (ec, stmt->parm.put.objref);
 	str = gb_value_promote (GB_EVAL_CONTEXT (ec), tmp, GB_VALUE_STRING);
 	gb_value_destroy (tmp);
@@ -549,11 +646,12 @@ gbrun_stmt_put (GBRunEvalContext *ec, const GBStatement *stmt)
 	}
 
 	gb_value_destroy (str);
-	
+	g_free (h);
+
 	return TRUE;
 }
 
-gboolean 
+gboolean
 gbrun_stmt_seek (GBRunEvalContext *ec, const GBStatement *stmt)
 {
 	GBValue          *handle;
@@ -562,7 +660,7 @@ gbrun_stmt_seek (GBRunEvalContext *ec, const GBStatement *stmt)
 
 	FILE_STMT_SEC_CHECK (ec, "seek within");
 
-	handle = gbrun_eval_as (ec, stmt->parm.seek.handle,  
+	handle = gbrun_eval_as (ec, stmt->parm.seek.handle,
 				GB_VALUE_INT);
 
 	h = internal_handle_from_gb_no (ec, handle->v.i);
@@ -574,7 +672,7 @@ gbrun_stmt_seek (GBRunEvalContext *ec, const GBStatement *stmt)
 		gbrun_exception_fire (ec, "Can't seek to a non-positive position");
 		return FALSE;
 	}
-		
+
 	if (h->mode != GB_OPEN_RANDOM) {
 		if (fseek (h->file, pos->v.l - 1, SEEK_SET)) {
 			gbrun_exception_fire (ec, "Unable to seek");
@@ -586,7 +684,15 @@ gbrun_stmt_seek (GBRunEvalContext *ec, const GBStatement *stmt)
 			return FALSE;
 		}
 	}
-	
+
+	g_free (h);
 	return TRUE;
 
 }
+
+void
+gbrun_files_init (GBRunEvalContext *ec)
+{
+	ec->file_handles = g_new0 (GBRunFileHandle, 512);
+
+}
