@@ -26,11 +26,17 @@ gbrun_objects_register (GBEvalContext *ec)
 	gbrun_listbox_register ();
 	gbrun_textbox_register ();
 	gbrun_timer_register ();
+	gbrun_picturebox_register ();
+	gbrun_frame_register();
+	gbrun_check_register();
 }
 
 void
 gbrun_objects_shutdown (void)
 {
+	gbrun_check_shutdown ();
+	gbrun_frame_shutdown ();
+	gbrun_picturebox_shutdown ();
 	gbrun_timer_shutdown ();
 	gbrun_textbox_shutdown ();
 	gbrun_listbox_shutdown ();
