@@ -17,12 +17,19 @@ void gbrun_label_register     (void);
 void gbrun_listbox_register   (void);
 void gbrun_textbox_register   (void);
 void gbrun_timer_register     (void);
+void gbrun_picturebox_register(void);
+void gbrun_frame_register     (void);
+void gbrun_check_register     (void);
 
 void gbrun_cmdbutton_shutdown (void);
 void gbrun_label_shutdown     (void);
 void gbrun_listbox_shutdown   (void);
 void gbrun_textbox_shutdown   (void);
 void gbrun_timer_shutdown     (void);
+void gbrun_picturebox_shutdown(void);
+void gbrun_check_shutdown     (void);
+void gbrun_frame_shutdown     (void);
+
 
 /* External */
 void gbrun_objects_register   (GBEvalContext *ec);
